@@ -3,29 +3,32 @@ use std::process::exit;
 
 use console::{pad_str, style, Alignment};
 use indenter::indented;
+use indoc::formatdoc;
 use itertools::Itertools;
+use rayon::prelude::*;
 
+use crate::backend::BackendType;
 use crate::build_time::built_info;
 use crate::cli::version;
 use crate::cli::version::VERSION;
 use crate::config::{Config, Settings};
 use crate::file::display_path;
 use crate::git::Git;
+use crate::plugins::core::CORE_PLUGINS;
 use crate::plugins::PluginType;
 use crate::shell::ShellType;
-use crate::toolset::Toolset;
-use crate::toolset::ToolsetBuilder;
+use crate::toolset::{Toolset, ToolsetBuilder};
 use crate::ui::style;
-use crate::{cmd, dirs, forge};
-use crate::{duration, env};
-use crate::{file, shims};
+use crate::{backend, cmd, dirs, duration, env, file, shims};
 
 /// Check mise installation for possible problems
 #[derive(Debug, clap::Args)]
 #[clap(visible_alias = "dr", verbatim_doc_comment, after_long_help = AFTER_LONG_HELP)]
 pub struct Doctor {
     #[clap(skip)]
-    checks: Vec<String>,
+    errors: Vec<String>,
+    #[clap(skip)]
+    warnings: Vec<String>,
 }
 
 impl Doctor {
@@ -40,26 +43,41 @@ impl Doctor {
 
         match Config::try_get() {
             Ok(config) => self.analyze_config(config)?,
-            Err(err) => self.checks.push(format!("failed to load config: {err}")),
+            Err(err) => self.errors.push(format!("failed to load config: {err}")),
         }
 
+        self.analyze_plugins();
+
         section("env_vars", mise_env_vars())?;
         self.analyze_settings()?;
 
         if let Some(latest) = version::check_for_new_version(duration::HOURLY) {
-            self.checks.push(format!(
+            self.errors.push(format!(
                 "new mise version {latest} available, currently on {}",
                 *version::V
             ));
         }
 
-        if self.checks.is_empty() {
+        if self.warnings.is_empty() {
+            miseprintln!("No warnings found");
+        } else {
+            let warnings_plural = if self.warnings.len() == 1 { "" } else { "s" };
+            let warning_summary =
+                format!("{} warning{warnings_plural} found:", self.warnings.len());
+            miseprintln!("{}\n", style(warning_summary).yellow().bold());
+            for (i, check) in self.warnings.iter().enumerate() {
+                let num = style::nyellow(format!("{}.", i + 1));
+                miseprintln!("{num} {}\n", indent_by(check, "   ").trim_start());
+            }
+        }
+
+        if self.errors.is_empty() {
             miseprintln!("No problems found");
         } else {
-            let checks_plural = if self.checks.len() == 1 { "" } else { "s" };
-            let summary = format!("{} problem{checks_plural} found:", self.checks.len());
-            miseprintln!("{}\n", style(summary).red().bold());
-            for (i, check) in self.checks.iter().enumerate() {
+            let errors_plural = if self.errors.len() == 1 { "" } else { "s" };
+            let error_summary = format!("{} problem{errors_plural} found:", self.errors.len());
+            miseprintln!("{}\n", style(error_summary).red().bold());
+            for (i, check) in self.errors.iter().enumerate() {
                 let num = style::nred(format!("{}.", i + 1));
                 miseprintln!("{num} {}\n", indent_by(check, "   ").trim_start());
             }
@@ -74,7 +92,7 @@ impl Doctor {
             Ok(settings) => {
                 section("settings", settings)?;
             }
-            Err(err) => self.checks.push(format!("failed to load settings: {err}")),
+            Err(err) => self.errors.push(format!("failed to load settings: {err}")),
         }
         Ok(())
     }
@@ -82,11 +100,12 @@ impl Doctor {
         let config = config.as_ref();
 
         section("config_files", render_config_files(config))?;
+        section("backends", render_backends())?;
         section("plugins", render_plugins())?;
 
-        for plugin in forge::list() {
+        for plugin in backend::list() {
             if !plugin.is_installed() {
-                self.checks
+                self.errors
                     .push(format!("plugin {} is not installed", &plugin.id()));
                 continue;
             }
@@ -95,8 +114,8 @@ impl Doctor {
         if !env::is_activated() && !shims_on_path() {
             let cmd = style::nyellow("mise help activate");
             let url = style::nunderline("https://mise.jdx.dev");
-            let shims = style::ncyan(dirs::SHIMS.display());
-            self.checks.push(formatdoc!(
+            let shims = style::ncyan(display_path(*dirs::SHIMS));
+            self.errors.push(formatdoc!(
                 r#"mise is not activated, run {cmd} or
                     read documentation at {url} for activation instructions.
                     Alternatively, add the shims directory {shims} to PATH.
@@ -109,7 +128,7 @@ impl Doctor {
                 self.analyze_shims(&ts);
                 self.analyze_toolset(&ts)?;
             }
-            Err(err) => self.checks.push(format!("failed to load toolset: {}", err)),
+            Err(err) => self.errors.push(format!("failed to load toolset: {}", err)),
         }
 
         Ok(())
@@ -141,32 +160,46 @@ impl Doctor {
     }
 
     fn analyze_shims(&mut self, toolset: &Toolset) {
+        let start_ms = std::time::Instant::now();
         let mise_bin = file::which("mise").unwrap_or(env::MISE_BIN.clone());
 
         if let Ok((missing, extra)) = shims::get_shim_diffs(mise_bin, toolset) {
             let cmd = style::nyellow("mise reshim");
 
             if !missing.is_empty() {
-                self.checks.push(formatdoc!(
+                self.errors.push(formatdoc!(
                     "shims are missing, run {cmd} to create them
                      Missing shims: {missing}",
-                    missing = missing.join(", ")
+                    missing = missing.into_iter().join(", ")
                 ));
             }
 
             if !extra.is_empty() {
-                self.checks.push(formatdoc!(
+                self.errors.push(formatdoc!(
                     "unused shims are present, run {cmd} to remove them
                      Unused shims: {extra}",
-                    extra = extra.join(", ")
+                    extra = extra.into_iter().join(", ")
                 ));
             }
         }
+        trace!("Shim analysis took {:?}", start_ms.elapsed());
+    }
+
+    fn analyze_plugins(&mut self) {
+        for plugin in backend::list() {
+            let is_core = CORE_PLUGINS.iter().any(|fg| fg.id() == plugin.id());
+            let plugin_type = plugin.get_plugin_type();
+
+            if is_core && plugin_type == PluginType::Asdf {
+                self.warnings
+                    .push(format!("plugin {} overrides a core plugin", &plugin.id()));
+            }
+        }
     }
 }
 
 fn shims_on_path() -> bool {
-    env::PATH.contains(&*dirs::SHIMS)
+    env::PATH.contains(&dirs::SHIMS.to_path_buf())
 }
 
 fn yn(b: bool) -> String {
@@ -183,7 +216,7 @@ fn mise_dirs() -> String {
         ("config", &*dirs::CONFIG),
         ("cache", &*dirs::CACHE),
         ("state", &*dirs::STATE),
-        ("shims", &dirs::SHIMS.as_path()),
+        ("shims", &*dirs::SHIMS),
     ]
     .iter()
     .map(|(k, p)| format!("{k}: {}", display_path(p)))
@@ -209,11 +242,21 @@ fn render_config_files(config: &Config) -> String {
         .join("\n")
 }
 
-fn render_plugins() -> String {
+fn render_backends() -> String {
     let mut s = vec![];
-    let plugins = forge::list()
+    let backends = backend::list_backend_types()
         .into_iter()
-        .filter(|p| p.is_installed())
+        .filter(|f| *f != BackendType::Asdf);
+    for b in backends {
+        s.push(format!("{}", b));
+    }
+    s.join("\n")
+}
+
+fn render_plugins() -> String {
+    let plugins = backend::list()
+        .into_iter()
+        .filter(|p| p.is_installed() && p.get_type() == BackendType::Asdf)
         .collect::<Vec<_>>();
     let max_plugin_name_len = plugins
         .iter()
@@ -221,26 +264,29 @@ fn render_plugins() -> String {
         .max()
         .unwrap_or(0)
         .min(40);
-    for p in plugins {
-        let padded_name = pad_str(p.id(), max_plugin_name_len, Alignment::Left, None);
-        let extra = match p.get_plugin_type() {
-            PluginType::External => {
-                let git = Git::new(dirs::PLUGINS.join(p.id()));
-                match git.get_remote_url() {
-                    Some(url) => {
-                        let sha = git
-                            .current_sha_short()
-                            .unwrap_or_else(|_| "(unknown)".to_string());
-                        format!("{url}#{sha}")
+    plugins
+        .into_par_iter()
+        .map(|p| {
+            let padded_name = pad_str(p.id(), max_plugin_name_len, Alignment::Left, None);
+            let extra = match p.get_plugin_type() {
+                PluginType::Asdf => {
+                    let git = Git::new(dirs::PLUGINS.join(p.id()));
+                    match git.get_remote_url() {
+                        Some(url) => {
+                            let sha = git
+                                .current_sha_short()
+                                .unwrap_or_else(|_| "(unknown)".to_string());
+                            format!("{url}#{sha}")
+                        }
+                        None => "".to_string(),
                     }
-                    None => "".to_string(),
                 }
-            }
-            PluginType::Core => "(core)".to_string(),
-        };
-        s.push(format!("{padded_name}  {}", style::ndim(extra)));
-    }
-    s.join("\n")
+                PluginType::Core => "(core)".to_string(),
+            };
+            format!("{padded_name}  {}", style::ndim(extra))
+        })
+        .collect::<Vec<_>>()
+        .join("\n")
 }
 
 fn build_info() -> String {
