@@ -1,11 +1,10 @@
 use eyre::Result;
+use itertools::Itertools;
+use std::collections::HashSet;
 
-use crate::cli::args::ToolArg;
+use crate::cli::args::{BackendArg, ToolArg};
 use crate::config::Config;
-use crate::forge;
-use crate::toolset::{
-    InstallOptions, ToolVersion, ToolVersionOptions, ToolVersionRequest, Toolset, ToolsetBuilder,
-};
+use crate::toolset::{InstallOptions, ToolRequest, ToolVersion, ToolVersionOptions, Toolset};
 use crate::ui::multi_progress_report::MultiProgressReport;
 
 /// Install a tool version
@@ -52,18 +51,19 @@ impl Install {
         match &self.tool {
             Some(runtime) => self.install_runtimes(&config, runtime)?,
             None => self.install_missing_runtimes(&config)?,
-        }
+        };
 
         Ok(())
     }
-    fn install_runtimes(&self, config: &Config, runtimes: &[ToolArg]) -> Result<()> {
+    fn install_runtimes(&self, config: &Config, runtimes: &[ToolArg]) -> Result<Vec<ToolVersion>> {
         let mpr = MultiProgressReport::get();
-        let mut ts = ToolsetBuilder::new().build(config)?;
-        let tool_versions = self.get_requested_tool_versions(&ts, runtimes, &mpr)?;
+        let tools: HashSet<BackendArg> = runtimes.iter().map(|ta| ta.backend.clone()).collect();
+        let mut ts = config.get_tool_request_set()?.filter_by_tool(&tools).into();
+        let tool_versions = self.get_requested_tool_versions(&ts, runtimes)?;
         if tool_versions.is_empty() {
             warn!("no runtimes to install");
             warn!("specify a version with `mise install <PLUGIN>@<VERSION>`");
-            return Ok(());
+            return Ok(vec![]);
         }
         ts.install_versions(config, tool_versions, &mpr, &self.install_opts())
     }
@@ -81,51 +81,52 @@ impl Install {
         &self,
         ts: &Toolset,
         runtimes: &[ToolArg],
-        mpr: &MultiProgressReport,
-    ) -> Result<Vec<ToolVersion>> {
+    ) -> Result<Vec<ToolRequest>> {
         let mut requests = vec![];
         for ta in ToolArg::double_tool_condition(runtimes)? {
             let default_opts = ToolVersionOptions::new();
             match ta.tvr {
-                Some(tv) => requests.push((ta.forge, tv, default_opts.clone())),
+                // user provided an explicit version
+                // TODO: this should install using options from config if the version matches
+                Some(tv) => requests.push(tv),
                 None => {
                     if ta.tvr.is_none() {
-                        match ts.versions.get(&ta.forge) {
+                        match ts.versions.get(&ta.backend) {
+                            // the tool is in config so fetch the params from config
+                            // this may match multiple versions of one tool (e.g.: python)
                             Some(tvl) => {
-                                for (tvr, opts) in &tvl.requests {
-                                    requests.push((ta.forge.clone(), tvr.clone(), opts.clone()));
+                                for tvr in &tvl.requests {
+                                    requests.push(tvr.clone());
                                 }
                             }
+                            // in this case the user specified a tool which is not in config
+                            // so we default to @latest with no options
                             None => {
-                                let tvr =
-                                    ToolVersionRequest::Version(ta.forge.clone(), "latest".into());
-                                requests.push((ta.forge, tvr, default_opts.clone()));
+                                let tvr = ToolRequest::Version {
+                                    backend: ta.backend.clone(),
+                                    version: "latest".into(),
+                                    options: default_opts.clone(),
+                                };
+                                requests.push(tvr);
                             }
                         }
                     }
                 }
             }
         }
-        let mut tool_versions = vec![];
-        for (fa, tvr, opts) in requests {
-            let plugin = forge::get(&fa);
-            plugin.ensure_installed(mpr, false)?;
-            let tv = tvr.resolve(plugin.as_ref(), opts, true)?;
-            tool_versions.push(tv);
-        }
-        Ok(tool_versions)
+        Ok(requests)
     }
 
-    fn install_missing_runtimes(&self, config: &Config) -> Result<()> {
-        let mut ts = ToolsetBuilder::new().build(config)?;
-        let versions = ts.list_missing_versions();
+    fn install_missing_runtimes(&self, config: &Config) -> eyre::Result<Vec<ToolVersion>> {
+        let trs = config.get_tool_request_set()?;
+        let versions = trs.missing_tools().into_iter().cloned().collect_vec();
         if versions.is_empty() {
             info!("all runtimes are installed");
-            return Ok(());
+            return Ok(vec![]);
         }
         let mpr = MultiProgressReport::get();
-        ts.install_versions(config, versions, &mpr, &self.install_opts())?;
-        Ok(())
+        let mut ts = Toolset::from(trs.clone());
+        ts.install_versions(config, versions, &mpr, &self.install_opts())
     }
 }
 
@@ -141,26 +142,33 @@ static AFTER_LONG_HELP: &str = color_print::cstr!(
 
 #[cfg(test)]
 mod tests {
+    use pretty_assertions::assert_str_eq;
+
     use crate::dirs;
+    use crate::test::reset;
 
     #[test]
     fn test_install_force() {
+        reset();
         assert_cli!("install", "-f", "tiny");
     }
 
     #[test]
     fn test_install_asdf_style() {
+        reset();
         assert_cli!("install", "tiny", "2");
     }
 
     #[test]
     fn test_install_with_alias() {
+        reset();
         assert_cli!("install", "-f", "tiny@my/alias");
         assert_cli_snapshot!("where", "tiny@my/alias");
     }
 
     #[test]
     fn test_install_ref() {
+        reset();
         assert_cli!("install", "-f", "dummy@ref:master");
         assert_cli!("global", "dummy@ref:master");
         let output = assert_cli!("where", "dummy");
@@ -173,6 +181,7 @@ mod tests {
 
     #[test]
     fn test_install_nothing() {
+        reset();
         // this doesn't do anything since dummy isn't specified
         assert_cli_snapshot!("install", "dummy");
     }
