@@ -4,19 +4,19 @@ use std::path::PathBuf;
 use std::sync::Arc;
 
 use console::style;
-use eyre::Result;
+use eyre::{ensure, Result};
 use indexmap::IndexMap;
 use itertools::Itertools;
 use serde_derive::Serialize;
 use tabled::{Table, Tabled};
 use versions::Versioning;
 
-use crate::cli::args::ForgeArg;
+use crate::backend::Backend;
+use crate::cli::args::BackendArg;
 use crate::config::Config;
-use crate::forge;
-use crate::forge::Forge;
-use crate::toolset::{ToolSource, ToolVersion, ToolsetBuilder};
+use crate::toolset::{ToolSource, ToolVersion, Toolset};
 use crate::ui::table;
+use crate::{backend, config};
 
 /// List installed and active tool versions
 ///
@@ -30,10 +30,10 @@ use crate::ui::table;
 pub struct Ls {
     /// Only show tool versions from [PLUGIN]
     #[clap(conflicts_with = "plugin_flag")]
-    plugin: Option<Vec<ForgeArg>>,
+    plugin: Option<Vec<BackendArg>>,
 
     #[clap(long = "plugin", short, hide = true)]
-    plugin_flag: Option<ForgeArg>,
+    plugin_flag: Option<BackendArg>,
 
     /// Only show tool versions currently specified in a .tool-versions/.mise.toml
     #[clap(long, short)]
@@ -105,7 +105,7 @@ impl Ls {
         match &self.plugin {
             Some(plugins) => {
                 for fa in plugins {
-                    let plugin = forge::get(fa);
+                    let plugin = backend::get(fa);
                     ensure!(plugin.is_installed(), "{fa} is not installed");
                 }
             }
@@ -129,7 +129,7 @@ impl Ls {
         let mut plugins = JSONOutput::new();
         for (plugin_name, runtimes) in &runtimes
             .into_iter()
-            .group_by(|(p, _, _)| p.id().to_string())
+            .chunk_by(|(p, _, _)| p.id().to_string())
         {
             let runtimes = runtimes.map(|row| row.into()).collect();
             plugins.insert(plugin_name.clone(), runtimes);
@@ -151,7 +151,7 @@ impl Ls {
                 // only displaying 1 plugin so only show the version
                 miseprintln!("{}", tv.version);
             } else {
-                miseprintln!("{} {}", &tv.forge, tv.version);
+                miseprintln!("{} {}", &tv.backend, tv.version);
             }
         }
         Ok(())
@@ -178,10 +178,21 @@ impl Ls {
     }
 
     fn get_runtime_list(&self, config: &Config) -> Result<Vec<RuntimeRow>> {
-        let tsb = ToolsetBuilder::new().with_global_only(self.global);
+        let mut trs = config.get_tool_request_set()?.clone();
+        if self.global {
+            trs = trs
+                .iter()
+                .filter(|(.., ts)| match ts {
+                    ToolSource::MiseToml(p) => config::is_global_config(p),
+                    _ => false,
+                })
+                .map(|(fa, tv, ts)| (fa.clone(), tv.clone(), ts.clone()))
+                .collect()
+        }
 
-        let ts = tsb.build(config)?;
-        let mut versions: HashMap<(String, String), (Arc<dyn Forge>, ToolVersion)> = ts
+        let mut ts = Toolset::from(trs);
+        ts.resolve()?;
+        let mut versions: HashMap<(String, String), (Arc<dyn Backend>, ToolVersion)> = ts
             .list_installed_versions()?
             .into_iter()
             .map(|(p, tv)| ((p.id().into(), tv.version.clone()), (p, tv)))
@@ -191,7 +202,7 @@ impl Ls {
             .list_current_versions()
             .into_iter()
             .map(|(p, tv)| ((p.id().into(), tv.version.clone()), (p, tv)))
-            .collect::<HashMap<(String, String), (Arc<dyn Forge>, ToolVersion)>>();
+            .collect::<HashMap<(String, String), (Arc<dyn Backend>, ToolVersion)>>();
 
         versions.extend(active.clone());
 
@@ -210,7 +221,7 @@ impl Ls {
             })
             .map(|(k, (p, tv))| {
                 let source = match &active.get(&k) {
-                    Some((_, tv)) => ts.versions.get(&tv.forge).map(|tv| tv.source.clone()),
+                    Some((_, tv)) => ts.versions.get(&tv.backend).map(|tv| tv.source.clone()),
                     None => None,
                 };
                 (p, tv, source)
@@ -218,7 +229,7 @@ impl Ls {
             // if it isn't installed and it's not specified, don't show it
             .filter(|(p, tv, source)| source.is_some() || p.is_version_installed(tv))
             .filter(|(p, _, _)| match &self.plugin {
-                Some(forge) => forge.contains(p.fa()),
+                Some(backend) => backend.contains(p.fa()),
                 None => true,
             })
             .collect();
@@ -243,13 +254,13 @@ struct JSONToolVersion {
     active: bool,
 }
 
-type RuntimeRow = (Arc<dyn Forge>, ToolVersion, Option<ToolSource>);
+type RuntimeRow = (Arc<dyn Backend>, ToolVersion, Option<ToolSource>);
 
 #[derive(Tabled)]
 #[tabled(rename_all = "PascalCase")]
 struct Row {
     #[tabled(display_with = "Self::display_plugin")]
-    plugin: Arc<dyn Forge>,
+    plugin: Arc<dyn Backend>,
     version: VersionStatus,
     #[tabled(rename = "Config Source", display_with = "Self::display_source")]
     source: Option<ToolSource>,
@@ -264,7 +275,7 @@ impl Row {
             None => String::new(),
         }
     }
-    fn display_plugin(plugin: &Arc<dyn Forge>) -> String {
+    fn display_plugin(plugin: &Arc<dyn Backend>) -> String {
         style(plugin).blue().to_string()
     }
     fn display_source(source: &Option<ToolSource>) -> String {
@@ -298,8 +309,8 @@ enum VersionStatus {
     Symlink(String, bool),
 }
 
-impl From<(&dyn Forge, &ToolVersion, &Option<ToolSource>)> for VersionStatus {
-    fn from((p, tv, source): (&dyn Forge, &ToolVersion, &Option<ToolSource>)) -> Self {
+impl From<(&dyn Backend, &ToolVersion, &Option<ToolSource>)> for VersionStatus {
+    fn from((p, tv, source): (&dyn Backend, &ToolVersion, &Option<ToolSource>)) -> Self {
         if p.symlink_path(tv).is_some() {
             VersionStatus::Symlink(tv.version.clone(), source.is_some())
         } else if !p.is_version_installed(tv) {
@@ -381,12 +392,16 @@ static AFTER_LONG_HELP: &str = color_print::cstr!(
 
 #[cfg(test)]
 mod tests {
+    use pretty_assertions::assert_str_eq;
+
     use crate::dirs;
     use crate::file::remove_all;
+    use crate::test::reset;
 
     #[test]
     fn test_ls() {
-        let _ = remove_all(dirs::INSTALLS.as_path());
+        reset();
+        let _ = remove_all(*dirs::INSTALLS);
         assert_cli!("install");
         assert_cli_snapshot!("list", @r###"
         dummy  ref:master  ~/.test-tool-versions     ref:master
@@ -422,6 +437,7 @@ mod tests {
 
     #[test]
     fn test_ls_current() {
+        reset();
         assert_cli_snapshot!("ls", "-c", @r###"
         dummy  ref:master  ~/.test-tool-versions     ref:master
         tiny   3.1.0       ~/cwd/.test-tool-versions 3
@@ -430,7 +446,8 @@ mod tests {
 
     #[test]
     fn test_ls_json() {
-        let _ = remove_all(dirs::INSTALLS.as_path());
+        reset();
+        let _ = remove_all(*dirs::INSTALLS);
         assert_cli!("install");
         assert_cli_snapshot!("ls", "--json");
         assert_cli_snapshot!("ls", "--json", "tiny");
@@ -438,7 +455,8 @@ mod tests {
 
     #[test]
     fn test_ls_parseable() {
-        let _ = remove_all(dirs::INSTALLS.as_path());
+        reset();
+        let _ = remove_all(*dirs::INSTALLS);
         assert_cli!("install");
         assert_cli_snapshot!("ls", "--parseable", @r###"
         dummy ref:master
@@ -455,19 +473,28 @@ mod tests {
 
     #[test]
     fn test_ls_missing() {
+        reset();
         assert_cli!("install");
         assert_cli_snapshot!("ls", "--missing", @"");
     }
 
     #[test]
     fn test_ls_missing_plugin() {
+        reset();
         let err = assert_cli_err!("ls", "missing-plugin");
         assert_str_eq!(err.to_string(), r#"missing-plugin is not installed"#);
     }
 
     #[test]
     fn test_ls_prefix() {
+        reset();
         assert_cli!("install");
         assert_cli_snapshot!("ls", "--plugin=tiny", "--prefix=3", @"tiny  3.1.0  ~/cwd/.test-tool-versions 3");
     }
+
+    #[test]
+    fn test_global() {
+        reset();
+        assert_cli_snapshot!("ls", "--global", @"");
+    }
 }
