@@ -5,7 +5,7 @@ use std::hash::{Hash, Hasher};
 use std::io::{Read, Write};
 use std::path::Path;
 
-use eyre::Result;
+use eyre::{ensure, Result};
 use rayon::prelude::*;
 use sha2::{Digest, Sha256};
 
@@ -72,6 +72,12 @@ pub fn parse_shasums(text: &str) -> HashMap<String, String> {
 
 #[cfg(test)]
 mod tests {
+    use insta::assert_snapshot;
+    use pretty_assertions::assert_eq;
+    use test_log::test;
+
+    use crate::test::reset;
+
     use super::*;
 
     #[test]
@@ -81,6 +87,7 @@ mod tests {
 
     #[test]
     fn test_hash_sha256() {
+        reset();
         let path = Path::new(".test-tool-versions");
         let hash = file_hash_sha256(path).unwrap();
         assert_snapshot!(hash);
