@@ -58,9 +58,6 @@ mod toolset;
 mod ui;
 
 fn main() -> eyre::Result<()> {
-    #[cfg(windows)]
-    warn!("mise is supported on windows. Do not expect anything to work.");
-
     let args = env::args().collect_vec();
     color_eyre::install()?;
 
